import OGIS.*;
import java.util.*;
import org.omg.CORBA.*;

/**
    Test Feature Interfaces
 */

public class FeatureTest {

    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static boolean testQCFCF(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    QueryableContainerFeatureCollectionFactory qcfcfRef,
                    String     ftName,  // FeatureType name
                    int        revision, // 10:Rev1.0, 11:Rev1.1(draft)
                    FeatureCollectionFactory fcfRef,
                    String     fcfftName // FeatureType used in FCF
                    )
    {
        boolean result = true;
        panel.print("> testing QueryableContainerFeatureCollectionFactory ..");

        try {
            QueryableContainerFeatureCollection qcfcRef = null;
            NVPair[] nvs = new NVPair[0];
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            FeatureType  ftype = FeatureTypeHelper.bind(
                                          orb, ftName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            Feature[]     list  = new Feature[0];
            FeatureData[] fdata = new FeatureData[0];
            FeatureCollection fc = testFCF( orb, panel, fcfRef,
                                           fcfftName, revision );
            /////////////////////////////////////////
            // Queryable Container Feature Collection Factory
            /////////////////////////////////////////
          try {
            panel.print(">     QCFCF.create()");
            qcfcRef = qcfcfRef.create( ftype, nvs );
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          }

            //================================================================
            // Queryable Container Feature Collection
            //================================================================
            boolean b = testQCFC( orb, panel, qcfcRef, revision,
                                  fcfRef, fcfftName );

            // other QCFCF's methods
          try {
            panel.print(">     QCFCF.createFromCollection()");
            qcfcRef = qcfcfRef.createFromCollection( ftype, nvs, fc );
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

          try {
            panel.print(">     QCFCF.createFromSequence()");
            qcfcRef = qcfcfRef.createFromSequence( ftype, nvs, list );
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

          try {
            panel.print(">     QCFCF.createFromFeatureData()");
            qcfcRef = qcfcfRef.createFromFeatureData( ftype, nvs, fdata );
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.QueryableContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }


    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static boolean testQCFC(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    QueryableContainerFeatureCollection qcfcRef,
                    int        revision, // 10:Rev1.0, 11:Rev1.1(draft)
                    FeatureCollectionFactory fcfRef,
                    String     fcfftName // FeatureType used in FCF
                    )
    {
        boolean result = true;
        panel.print("> testing QueryableContainerFeatureCollection ..");

        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            FeatureType  ftype = FeatureTypeHelper.bind(
                                          orb, fcfftName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            // for creating FeatureCollection
            FeatureCollection fcRef = testFCF( orb, panel,  fcfRef,
                                               fcfftName, revision );

            //================================================================
            // Test methods inherited from ContainerFeatureCollection
            //================================================================
            boolean bbb = testCFC( orb, panel, qcfcRef, revision,
                                   fcfRef, fcfftName );

            if ( ! bbb ) { throw new Exception(); }
            //================================================================
            // Test methods inherited from QueryEvaluator
            //================================================================
            panel.print(">     ql_types()");
            OGIS.QueryEvaluatorPackage.QLType[] qls = qcfcRef.ql_types();

            panel.print(">     default_ql_type()");
            OGIS.QueryEvaluatorPackage.QLType   ql  = qcfcRef.default_ql_type();

            if ( revision == 10 )
            {
/*** ADAPTATION Rev1.0 ***/
              NVPair[] nvs = new NVPair[0];
              panel.print(">     evaluate()");
              QueryResultSetIterator qrsi =
                 qcfcRef.evaluate( "select * from feature", ql, nvs );

              Geometry g = null;
              String[] p = new String[0];
              panel.print(">     query_by_properties()");
              qrsi =
                 qcfcRef.query_by_properties( "", ql, g,
                 OGIS.QueryEvaluatorPackage.SpatialOperatorType.WithinOp, p );

              //==============================================================
              // QueryResultSetIterator
              //==============================================================
              {
                String name = "name";
                int    idx  = 1;
                panel.print(">     QueryResultSetIterator.advance()");
                qrsi.advance();

                panel.print(">     QueryResultSetIterator.get_record()");
                OGIS.QueryResultSetIteratorPackage.FieldValue[] r =
                    qrsi.get_record();

                panel.print(">     QueryResultSetIterator.get_record_as_feature()");
                Feature f = qrsi.get_record_as_feature(ftype);

                panel.print(">     QueryResultSetIterator.get_property_by_index()");
                OGIS.QueryResultSetIteratorPackage.Value   v =
                    qrsi.get_property_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_property_by_name()");
                        v = qrsi.get_property_by_name(name);

                panel.print(">     QueryResultSetIterator.get_string_by_index()");
                String  str = qrsi.get_string_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_string_by_name()");
                        str = qrsi.get_string_by_name(name);

                panel.print(">     QueryResultSetIterator.get_float_by_index()");
                float   fval = qrsi.get_float_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_float_by_name()");
                        fval = qrsi.get_float_by_name(name);

                panel.print(">     QueryResultSetIterator.get_double_by_index()");
                double  dval = qrsi.get_double_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_double_by_name()");
                        dval = qrsi.get_double_by_name(name);

                panel.print(">     QueryResultSetIterator.get_long_by_index()");
                int     ival = qrsi.get_long_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_long_by_name()");
                        ival = qrsi.get_long_by_name(name);

                panel.print(">     QueryResultSetIterator.get_short_by_index()");
                short   sval = qrsi.get_short_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_short_by_name()");
                        sval = qrsi.get_short_by_name(name);

                panel.print(">     QueryResultSetIterator.get_boolean_by_index()");
                boolean bool = qrsi.get_boolean_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_boolean_by_name()");
                        bool = qrsi.get_boolean_by_name(name);

                panel.print(">     QueryResultSetIterator.get_decimal_by_index()");
                OGIS.QueryResultSetIteratorPackage.Decimal deci =
                    qrsi.get_decimal_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_decimal_by_name()");
                        deci = qrsi.get_decimal_by_name(name);

                panel.print(">     QueryResultSetIterator.get_byte_stream_by_index()");
                byte[]  oct = qrsi.get_byte_stream_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_byte_stream_by_name()");
                        oct = qrsi.get_byte_stream_by_name(name);

                panel.print(">     QueryResultSetIterator.get_geometry_by_index()");
                WKSGeometry wksg = qrsi.get_geometry_by_index(idx);

                panel.print(">     QueryResultSetIterator.get_geometry_by_name()");
                            wksg = qrsi.get_geometry_by_name(name);

                panel.print(">     QueryResultSetIterator.get_metadata()");
                QueryResultSetMetaData qrsmd = qrsi.get_metadata();

                //============================================================
                // QueryResultSetMetaData
                //============================================================
                {
                  panel.print(">     QueryResultSetMetaData.get_record()");
                  long       n = qrsmd.get_property_count();

                  panel.print(">     QueryResultSetMetaData.get_schema_name()");
                  String    sn = qrsmd.get_schema_name(idx);

                  panel.print(">     QueryResultSetMetaData.get_property_name()");
                  String    pn = qrsmd.get_property_name(idx);

                  panel.print(">     QueryResultSetMetaData.get_property_type()");
                  OGIS.QueryResultSetIteratorPackage.ValueType vt =
                      qrsmd.get_property_type(idx);

                  panel.print(">     QueryResultSetMetaData.get_property_type_name()");
                  String ptname =
                      qrsmd.get_property_type_name(idx);

                  boolean  mdb;
                  panel.print(">     QueryResultSetMetaData.is_read_only()");
                  mdb = qrsmd.is_read_only(idx);

                  panel.print(">     QueryResultSetMetaData.is_writeable()");
                  mdb = qrsmd.is_writeable(idx);

                  panel.print(">     QueryResultSetMetaData.is_case_sensitive()");
                  mdb = qrsmd.is_case_sensitive(idx);

                  panel.print(">     QueryResultSetMetaData.is_searchable()");
                  mdb = qrsmd.is_searchable(idx);

                  panel.print(">     QueryResultSetMetaData.is_signed()");
                  mdb = qrsmd.is_signed(idx);

                  panel.print(">     QueryResultSetMetaData.is_currency()");
                  mdb = qrsmd.is_currency(idx);
                }
              }
              panel.print(">     QueryResultSetIterator.destroy()");
              qrsi.destroy();
/*** ADAPTATION Rev1.0 ***/
            }
            else if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                OGIS.QueryEvaluatorPackage.GeomConstraint[] geom =
                     new OGIS.QueryEvaluatorPackage.GeomConstraint[0];
                String query = "";
              try {
                panel.print(">     query()");
                QueryableFeatureCollection qfc = qcfcRef.query( query, ql, geom );
              } catch ( OGIS.QueryEvaluatorPackage.QueryLanguageTypeNotSupported e ) {
                panel.print(">     -- query language type not supported.");
              } catch ( OGIS.QueryEvaluatorPackage.InvalidQuery e ) {
                panel.print(">     -- invalid query.");
              } catch ( OGIS.QueryEvaluatorPackage.InvalidGeometry e ) {
                panel.print(">     -- invalid geometry.");
              } catch ( OGIS.QueryEvaluatorPackage.QueryProcessingError e ) {
                panel.print(">     -- query processing error.");
              } catch ( OGIS.QueryEvaluatorPackage.InvalidSpatialOperator e ) {
                panel.print(">     -- invalid spatial operator.");
              }

*** ADAPTATION Rev1.1 ***/
            }
        } catch (Exception e) {
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }


    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static FeatureCollection testFCF(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    FeatureCollectionFactory fcfRef,
                    String     ftName,  // FeatureType name
                    int        revision // 10:Rev1.0, 11:Rev1.1(draft)
                    )
    {
        boolean result = true;
        FeatureCollection fcRef = null;
        panel.print("> testing FeatureCollectionFactory ..");

        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            FeatureType       ftype = FeatureTypeHelper.bind(
                                          orb, ftName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            NVPair[]            nvs = new NVPair[0];
            Feature[]          list = new Feature[0];
            FeatureType[] supported = new FeatureType[0];
            //================================================================
            // FeatureCollectionFactory
            //================================================================
            if ( revision == 10 )
            {
/*** ADAPTATION Rev1.0 ***/
              try {
                panel.print(">     FCF.create()");
                fcRef = fcfRef.create( ftype, nvs );
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              }

              try {
                panel.print(">     FCF.createFromCollection()");
                fcRef = fcfRef.createFromCollection( ftype, nvs, fcRef );
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureInvalid e ) {
                panel.print(">     -- feature invalid.");
              }

              try {
                panel.print(">     FCF.createFromSequence()");
                fcRef = fcfRef.createFromSequence( ftype, nvs, list );
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureInvalid e ) {
                panel.print(">     -- feature invalid.");
              }

/*** ADAPTATION Rev1.0 ***/
            }
            else if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
              try {
                panel.print(">     FCF.create()");
                fcRef = fcfRef.create( ftype, nvs, supported );
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              }

              try {
                panel.print(">     FCF.createFromCollection()");
                fcRef = fcfRef.createFromCollection( ftype, nvs, supported, fcRef );
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureInvalid e ) {
                panel.print(">     -- feature invalid.");
              }

              try {
                panel.print(">     FCF.createFromSequence()");
                fcRef = fcfRef.createFromSequence( ftype, nvs, supported, list);
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
                panel.print(">     -- feature type invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.PropertyInvalid e ) {
                panel.print(">     -- property invalid.");
              } catch ( OGIS.FeatureCollectionFactoryPackage.FeatureInvalid e ) {
                panel.print(">     -- feature invalid.");
              }
*** ADAPTATION Rev1.1 ***/
            }
        } catch (Exception e) {
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        if ( fcRef == null ) {
            panel.print("> FeatureCollection = null !");
        }

        return fcRef;
    }

    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static boolean testFC(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    FeatureCollection fcRef,
                    int        revision // 10:Rev1.0, 11:Rev1.1(draft)
                    )
    {
        boolean result = true;
        panel.print("> testing FeatureCollection ..");

        try {
            //================================================================
            // Feature Collection
            //================================================================
            panel.print(">     number_features()");
            int n = fcRef.number_features();

            Feature[] fseq = new Feature[n];

            panel.print(">     FC.create_iterator()");
            FeatureIterator itr = fcRef.create_iterator();

            panel.print(">     FC.FeatureIterator.more()");
            panel.print(">     FC.FeatureIterator.next()");
            panel.print(">     FC.FeatureIterator.next_n()");
            int i = 0;
            boolean b = false;
            FeatureHolder    f_hold  = new FeatureHolder();
            FeatureSeqHolder fs_hold = new FeatureSeqHolder();
            while ( itr.more() ) {
                b = itr.next( f_hold );
                fseq[i] = f_hold.value;
                b = itr.next_n((short)(n-1), fs_hold);
                for ( int j=0; j < fs_hold.value.length; j++ ) {
                   fseq[j+1] = fs_hold.value[j];
                }
            }
            panel.print(">     FC.FeatureIterator.reset()");
            itr.reset();

            panel.print(">     FC.retrieve_element_at()");
            Feature f = fcRef.retrieve_element_at(itr);

            panel.print(">     FC.insert_element_at()");
            fcRef.insert_element_at(f, itr);

            panel.print(">     FC.replace_element_at()");
            fcRef.replace_element_at(f, itr);

            panel.print(">     FC.remove_element_at()");
            fcRef.remove_element_at(itr);

            panel.print(">     FC.remove_all_elements()");
            fcRef.remove_all_elements();

            panel.print(">     FC.add_element_at()");
            fcRef.add_element( f );

            panel.print(">     FC.merge()");
            fcRef.merge(fcRef);

            {
            // For Testing methods inherited from Feature
            panel.print(">     FC.feature_type()");
            FeatureType       ftyp = fcRef.feature_type();

          try {
            NVPair[] nvs = new NVPair[0];
            panel.print(">     FC.get_geometry()");
            Geometry          geom = fcRef.get_geometry(nvs);
          } catch ( OGIS.FeaturePackage.InvalidParams e ) {
            panel.print(">     -- invalid parameters.");
          }

            String name = "name";
            panel.print(">     FC.property_exists()");
            boolean           bool = fcRef.property_exists(name);

          try {
            panel.print(">     FC.get_property()");
            org.omg.CORBA.Any prop = fcRef.get_property(name);
          } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
            panel.print(">     -- property not set.");
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          }

          try {
            org.omg.CORBA.Any prop = orb.create_any();
            prop.insert_string("value");
            panel.print(">     FC.set_property()");
            fcRef.set_property(name, prop);
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          } catch ( OGIS.FeaturePackage.InvalidValue e ) {
            panel.print(">     -- invalid value.");
          }

            panel.print(">     FC.get_property_sequence()");
            NVPair[] nvss = fcRef.get_property_sequence(3);

          try {
            panel.print(">     FC.delete_property()");
            fcRef.delete_property(name);
          } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
            panel.print(">     -- property not set.");
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          } catch ( OGIS.FeaturePackage.RequiredProperty e ) {
            panel.print(">     -- required property.");
          }

            }

            if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                panel.print(">     FC.supported_feature_types()");
                FeatureType[] ftyps = fcRef.supported_feature_types();

                String fname = "name";
              try {
                panel.print(">     FC.get_string_by_name()");
                String      strg = fcRef.get_string_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_float_by_name()");
                float       fval = fcRef.get_float_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_double_by_name()");
                double      dval = fcRef.get_double_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_long_by_name()");
                int         lval = fcRef.get_long_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_short_by_name()");
                short       sval = fcRef.get_short_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_boolean_by_name()");
                boolean     bool = fcRef.get_boolean_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_decimal_by_name()");
                Decimal     deci = fcRef.get_decimal_by_name(fname);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.get_byte_stream_by_name()");
                byte[]      octe = fcRef.get_byte_stream_by_name(fname);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.get_geometry_by_name()");
                Geometry    geom = fcRef.get_geometry_by_name(fname);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.get_wksgeometry_by_name()");
                WKSGeometry wksg = fcRef.get_wksgeometry_by_name(fname);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.get_wkbgeometry_by_name()");
                byte[]      wkbg = fcRef.get_wkbgeometry_by_name(fname);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }
*** ADAPTATION Rev1.1 ***/
            }
            //================================================================
            //  Feature Iterator
            // Note: more(), next(), next_n() are already tested above.
            //================================================================
            {
            itr.reset();
            panel.print(">     FC.FeatureIterator.advance()");
            itr.advance();

            panel.print(">     FC.FeatureIterator.current()");
            Feature _f = itr.current();

            NVPair[] nvs = new NVPair[0];
            panel.print(">     FC.FeatureIterator.get_geometry()");
            Geometry g = itr.get_geometry(nvs);

            panel.print(">     FC.FeatureIterator.get_property()");
            org.omg.CORBA.Any any = itr.get_property("name");

            FeaturePropertySetIterator pitr;
            if ( revision == 10 )
            {
/*** ADAPATATION Rev1.0 ***/
                panel.print(">     FC.FeatureIterator.get_properties_sequence()");
                nvs  = itr.get_properties_sequence(1);

                panel.print(">     FC.FeatureIterator.get_properties_iterator()");
                pitr = itr.get_properties_iterator();
/*** ADAPTATION Rev1.0 ***/
            }
            else if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                panel.print(">     FC.FeatureIterator.get_property_sequence()");
                nvs = itr.get_property_sequence(1);

                panel.print(">     FC.FeatureIterator.get_property_iterator()");
                pitr = itr.get_property_iterator();

                panel.print(">     FC.FeatureIterator.get_feature_type()");
                FeatureType ft = itr.get_feature_type();

                String name = "name";

                panel.print(">     FC.FeatureIterator.property_exists()");
                boolean     ex = itr.property_exists(name);

              try {
                panel.print(">     FC.FeatureIterator.get_string_by_name()");
                String      ss = itr.get_string_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_float_by_name()");
                float       ff = itr.get_float_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_double_by_name()");
                double      dd = itr.get_double_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_long_by_name()");
                int         ii = itr.get_long_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_short_by_name()");
                short       sh = itr.get_short_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_boolean_by_name()");
                boolean     bb = itr.get_boolean_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_decimal_by_name()");
                Decimal     dc = itr.get_decimal_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_byte_stream_by_name()");
                byte[]      oc = itr.get_byte_stream_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_geometry_by_name()");
                Geometry    gg = itr.get_geometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_wksgeometry_by_name()");
                WKSGeometry ws = itr.get_wksgeometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FC.FeatureIterator.get_wkbgeometry_by_name()");
                byte[]      wb = itr.get_wkbgeometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }
*** ADAPTATION Rev1.1 ***/
            }

            panel.print(">     FC.FeatureIterator.destroy()");
            itr.destroy();    // destroy FeatureIterator
            }

            panel.print(">     FC.destroy()");
            fcRef.destroy();

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }

    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static boolean testCFCF(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    ContainerFeatureCollectionFactory cfcfRef,
                    String     ftName,  // FeatureType name
                    int        revision, // 10:Rev1.0, 11:Rev1.1(draft)
                    FeatureCollectionFactory fcfRef,
                    String     fcfftName // FeatureType used in FCF
                    )
    {
        boolean result = true;
        panel.print("> testing ContainerFeatureCollectionFactory ..");

        try {
            ContainerFeatureCollection cfcRef = null;
            NVPair[] nvs = new NVPair[0];

/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            FeatureType  ftype = FeatureTypeHelper.bind(
                                          orb, ftName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            Feature[]     list  = new Feature[0];
            FeatureData[] fdata = new FeatureData[0];
            FeatureCollection fc = testFCF( orb, panel, fcfRef,
                                           ftName, revision );

            //================================================================
            // Container Feature Collection Factory
            //================================================================
          try {
            panel.print(">     CFCF.create()");
            cfcRef = cfcfRef.create( ftype, nvs );
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          }

          try {
            panel.print(">     CFCF.createFromCollection()");
            cfcRef = cfcfRef.createFromCollection( ftype, nvs, fc );
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

          try {
            panel.print(">     CFCF.createFromSequence()");
            cfcRef = cfcfRef.createFromSequence( ftype, nvs, list );
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

          try {
            panel.print(">     CFCF.createFromFeatureData()");
            cfcRef = cfcfRef.createFromFeatureData( ftype, nvs, fdata );
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.PropertyInvalid e ) {
            panel.print(">     -- property invalid.");
          } catch ( OGIS.ContainerFeatureCollectionFactoryPackage.FeatureInvalid e ) {
            panel.print(">     -- feature invalid.");
          }

            if ( cfcRef == null ) {
                result = false;
                panel.print(">     Could not create CFC and continue.");
                throw new Exception();
            }
            //================================================================
            // ContainerFeatureCollection
            //================================================================
            boolean b = testCFC( orb, panel, cfcRef, revision,
                                 fcfRef, fcfftName );

        } catch (Exception e) {
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }

    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public static boolean testCFC(
                    org.omg.CORBA.ORB orb,
                    MainPanel  panel,
                    ContainerFeatureCollection cfcRef,
                    int        revision, // 10:Rev1.0, 11:Rev1.1(draft)
                    FeatureCollectionFactory fcfRef,
                    String     fcfftName // FeatureType used in FCF
                    )
    {
        boolean result = true;
        panel.print("> testing ContainerFeatureCollection ..");

        try {

            // for creating FeatureCollection
            FeatureCollection fcRef = testFCF( orb, panel,  fcfRef,
                                               fcfftName, revision );

            //================================================================
            // Container Feature Collection
            //================================================================
            panel.print(">     number_features()");
            int n = cfcRef.number_features();

            Feature[] fseq = new Feature[n];

            panel.print(">     create_iterator()");
            FeatureIterator itr = cfcRef.create_iterator();

            panel.print(">     FeatureIterator.more()");
            panel.print(">     FeatureIterator.next()");
            panel.print(">     FeatureIterator.next_n()");
            int i = 0;
            boolean b = false;
            FeatureHolder    f_hold  = new FeatureHolder();
            FeatureSeqHolder fs_hold = new FeatureSeqHolder();
            while ( itr.more() ) {
                b = itr.next( f_hold );
                fseq[i] = f_hold.value;
                b = itr.next_n((short)(n-1), fs_hold);
                for ( int j=0; j < fs_hold.value.length; j++ ) {
                   fseq[j+1] = fs_hold.value[j];
                }
            }
            panel.print(">     FeatureIterator.reset()");
            itr.reset();

            panel.print(">     retrieve_element_at()");
            Feature f = cfcRef.retrieve_element_at(itr);

            // If CFC is read only, follow methods throw exceptions.

            panel.print(">     insert_element_at()");
            cfcRef.insert_element_at(f, itr);

            panel.print(">     replace_element_at()");
            cfcRef.replace_element_at(f, itr);

            panel.print(">     remove_element_at()");
            cfcRef.remove_element_at(itr);

            panel.print(">     remove_all_elements()");
            cfcRef.remove_all_elements();

            panel.print(">     add_element_at()");
            cfcRef.add_element( f );

            panel.print(">     merge()");
            cfcRef.merge(fcRef);


            {   //--------------------------------
            //
            // for testing methods inherited from Feature
            //
            panel.print(">     feature_type()");
            FeatureType       ftyp = cfcRef.feature_type();

            NVPair[] nvs = new NVPair[0];
          try {
            panel.print(">     get_geometry()");
            Geometry          geom = cfcRef.get_geometry(nvs);
          } catch ( OGIS.FeaturePackage.InvalidParams e ) {
            panel.print(">     -- invalid parameters.");
          }

            String name = "name";
            panel.print(">     property_exists()");
            boolean           bool = cfcRef.property_exists(name);

            org.omg.CORBA.Any prop1 = orb.create_any();
            prop1.insert_string("value");
          try {
            panel.print(">     set_property()");
            cfcRef.set_property(name, prop1);
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          } catch ( OGIS.FeaturePackage.InvalidValue e ) {
            panel.print(">     -- invalid value.");
          }

          try {
            panel.print(">     get_property()");
            org.omg.CORBA.Any prop2 = cfcRef.get_property(name);
          } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
            panel.print(">     -- property not set.");
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          }

            panel.print(">     get_property_sequence()");
            NVPair[] nvss = cfcRef.get_property_sequence(3);

          try {
            panel.print(">     delete_property()");
            cfcRef.delete_property(name);
          } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
            panel.print(">     -- property not set.");
          } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
            panel.print(">     -- invalid property.");
          } catch ( OGIS.FeaturePackage.RequiredProperty e ) {
            panel.print(">     -- required property.");
          }

            //
            // For testing methods inherited from FeatureFactory
            //

          try {
            panel.print(">     create_feature()");
            Feature   _f = cfcRef.create_feature( ftyp, nvs );
          } catch ( OGIS.FeatureFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid");
          } catch ( OGIS.FeatureFactoryPackage.PropertiesInvalid e ) {
            panel.print(">     -- properties invalid");
          }

            FeatureData[] fdats = new FeatureData[1];
            fdats[0] = new FeatureData(ftyp, nvs);
          try {
            panel.print(">     create_feature() (from FeatureData)");
            Feature[] fs = cfcRef.create_features(fdats);
          } catch ( OGIS.FeatureFactoryPackage.FeatureTypeInvalid e ) {
            panel.print(">     -- feature type invalid");
          } catch ( OGIS.FeatureFactoryPackage.PropertiesInvalid e ) {
            panel.print(">     -- properties invalid");
          }

            } //----------------------

            if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                panel.print(">     supported_feature_types()");
                FeatureType[] ftyps = cfcRef.supported_feature_types();

                String name = "name";
              //org.omg.CORBA.Any any1 = orb.create_any();
              //any1.insert_string("string_value");
              //cfcRef.set_property( name, any1 );
              try {
                panel.print(">     get_string_by_name()");
                String      strg = cfcRef.get_string_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_float_by_name()");
                float       fval = cfcRef.get_float_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_double_by_name()");
                double      dval = cfcRef.get_double_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_long_by_name()");
                int         lval = cfcRef.get_long_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_short_by_name()");
                short       sval = cfcRef.get_short_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_boolean_by_name()");
                boolean     bool = cfcRef.get_boolean_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              //OGIS.DecimalHelper.insert( any1,
              //    new OGIS.Decimal(0,0, new byte[1]) );
              //cfcRef.set_property( name, any1 );
              try {
                panel.print(">     get_decimal_by_name()");
                Decimal     deci = cfcRef.get_decimal_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              //byte[] bytes = new byte[10];
              //OGIS.OctetSeqHelper.insert( any1, bytes );
              //cfcRef.set_property( name, any1 );
              try {
                panel.print(">     get_byte_stream_by_name()");
                byte[]      octe = cfcRef.get_byte_stream_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     get_geometry_by_name()");
                Geometry    geom = cfcRef.get_geometry_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     get_wksgeometry_by_name()");
                WKSGeometry wksg = cfcRef.get_wksgeometry_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     get_wkbgeometry_by_name()");
                byte[]      wkbg = cfcRef.get_wkbgeometry_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

*** ADAPTATION Rev1.1 ***/
            }
            //================================================================
            //  Feature Iterator
            // Note: more(), next(), next_n() are already tested above.
            //================================================================
            {
            itr.reset();
            panel.print(">     FeatureIterator.advance()");
            itr.advance();

            panel.print(">     FeatureIterator.current()");
            Feature _f = itr.current();

            NVPair[] nvs = new NVPair[0];
            panel.print(">     FeatureIterator.get_geometry()");
            Geometry g = itr.get_geometry(nvs);

            panel.print(">     FeatureIterator.get_property()");
            org.omg.CORBA.Any any = itr.get_property("GID");

            FeaturePropertySetIterator pitr = null;
            if ( revision == 10 )
            {
/*** ADAPTATION Rev1.0 ***/
                panel.print(">     FeatureIterator.get_properties_sequence()");
                nvs  = itr.get_properties_sequence(1);

                panel.print(">     FeatureIterator.get_properties_iterator()");
                pitr = itr.get_properties_iterator();
/*** ADAPTATION Rev1.0 ***/
            }
            else if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                panel.print(">     FeatureIterator.get_property_sequence()");
                nvs = itr.get_property_sequence(1);

                panel.print(">     FeatureIterator.get_property_iterator()");
                pitr = itr.get_property_iterator();

                panel.print(">     FeatureIterator.get_feature_type()");
                FeatureType ft = itr.get_feature_type();

                String name = "COUNTRY";
                panel.print(">     FeatureIterator.property_exists()");
                boolean     ex = itr.property_exists(name);

              try {
                panel.print(">     FeatureIterator.get_string_by_name()");
                String      ss = itr.get_string_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_float_by_name()");
                float       ff = itr.get_float_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_double_by_name()");
                double      dd = itr.get_double_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_long_by_name()");
                int         ii = itr.get_long_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_short_by_name()");
                short       sh = itr.get_short_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_boolean_by_name()");
                boolean     bb = itr.get_boolean_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_decimal_by_name()");
                Decimal     dc = itr.get_decimal_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FeatureIterator.get_byte_stream_by_name()");
                byte[]      oc = itr.get_byte_stream_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FeatureIterator.get_geometry_by_name()");
                Geometry    gg = itr.get_geometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     FeatureIterator.get_wksgeometry_by_name()");
                WKSGeometry ws = itr.get_wksgeometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     FeatureIterator.get_wkbgeometry_by_name()");
                byte[]      wb = itr.get_wkbgeometry_by_name(name);
              } catch ( OGIS.FeatureIteratorPackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeatureIteratorPackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }
*** ADAPTATION Rev1.1 ***/
            }

            //================================================================
            //  PropertySetIterator
            //================================================================
            {
                NVPairHolder    nvhold  = new NVPairHolder();;
                NVPairSeqHolder nvshold = new NVPairSeqHolder();;
                boolean bb;
                panel.print(">     PropertySetIterator.next()");
                bb = pitr.next(nvhold);

                panel.print(">     PropertySetIterator.reset()");
                pitr.reset();

                panel.print(">     PropertySetIterator.next_n()");
                bb = pitr.next_n(2,nvshold);

                panel.print(">     PropertySetIterator.destroy()");
                pitr.destroy();
            }

            }
            //================================================================
            //  Feature
            //================================================================
            {
            itr.reset();
            Feature fet = itr.current();

            panel.print(">     Feature.feature_type()");
            FeatureType ftype = fet.feature_type();

            NVPair[] nvs = new NVPair[0];
            panel.print(">     Feature.get_geometry()");
            Geometry     g = fet.get_geometry(nvs);

            String pname = "name";
            panel.print(">     Feature.property_exists()");
            boolean     _b = fet.property_exists(pname);

            Any p1 = ORB.init().create_any();
            p1.insert_string("value");
            panel.print(">     Feature.set_property()");
            fet.set_property(pname, p1);

            panel.print(">     Feature.get_property()");
            Any          p = fet.get_property(pname);

            panel.print(">     Feature.get_property_sequence()");
            NVPair[] nv = fet.get_property_sequence(3);

            panel.print(">     Feature.delete_property()");
            fet.delete_property(pname);

            if ( revision == 11 )
            {
/*** ADAPTATION Rev1.1 ***
                String name = "WKB_GEOMETRY";
              try {
                panel.print(">     Feature.get_string_by_name()");
                String      strg = fet.get_string_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_float_by_name()");
                float       fval = fet.get_float_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_double_by_name()");
                double      dval = fet.get_double_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_long_by_name()");
                int         ival = fet.get_long_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_short_by_name()");
                short       sval = fet.get_short_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_boolean_by_name()");
                boolean     bool = fet.get_boolean_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- invalid property.");
              }

              try {
                panel.print(">     Feature.get_decimal_by_name()");
                Decimal     deci = fet.get_decimal_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     Feature.get_byte_stream_by_name()");
                byte[]      octe = fet.get_byte_stream_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     Feature.get_geometry_by_name()");
                Geometry    geom = fet.get_geometry_by_name(name);
              } catch ( OGIS.FeaturePackage.PropertyNotSet e ) {
                panel.print(">     -- property not set.");
              }

              try {
                panel.print(">     Feature.get_wksgeometry_by_name()");
                WKSGeometry wksg = fet.get_wksgeometry_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }

              try {
                panel.print(">     Feature.get_wkbgeometry_by_name()");
                byte[]      wkbg = fet.get_wkbgeometry_by_name(name);
              } catch ( OGIS.FeaturePackage.InvalidProperty e ) {
                panel.print(">     -- property not set.");
              } catch ( OGIS.FeaturePackage.InvalidConversion e ) {
                panel.print(">     -- invalid conversion.");
              }
*** ADAPTATION Rev1.1 ***/
            }
            //================================================================
            // FeatureType
            //================================================================
            {
            panel.print(">     FeatureType.name()");
            String          s = ftype.name();

            try {
              panel.print(">     FeatureType.get_parents()");
              FeatureType[] ft1 = ftype.get_parents();
            } catch ( OGIS.FeatureTypePackage.InheritanceUnsupported e ) {
              panel.print(">     -- Inheritance Unsupported.");
            }

            try {
              panel.print(">     FeatureType.get_children()");
              FeatureType[] ft2 = ftype.get_children();
            } catch ( OGIS.FeatureTypePackage.InheritanceUnsupported e ) {
              panel.print(">     -- Inheritance Unsupported.");
            }

            String pdname = "name";
            panel.print(">     FeatureType.property_def_exists()");
            boolean        bo = ftype.property_def_exists(pdname);

          try {
            panel.print(">     FeatureType.get_property_def()");
            PropertyDef    pd = ftype.get_property_def(pdname);
          } catch ( OGIS.FeatureTypePackage.PropertyDefInvalid e ) {
            panel.print(">     -- PropertyDef invalid.");
          }

            panel.print(">     FeatureType.get_property_def_sequence()");
            PropertyDef[] pds = ftype.get_property_def_sequence(-1,1);

            panel.print(">     FeatureType.get_property_def_iterator()");
            PropertyDefIterator pdi = ftype.get_property_def_iterator(-1);

            //================================================================
            // Test PropertyDefIterator
            //================================================================
            {
              boolean boo;
              PropertyDefHolder    pdhold  = new PropertyDefHolder();
              PropertyDefSeqHolder pdshold = new PropertyDefSeqHolder();

              panel.print(">     PropertyDefIterator.next()");
              boo = pdi.next(pdhold);

              panel.print(">     PropertyDefIterator.reset()");
              pdi.reset();

              panel.print(">     PropertyDefIterator.next_n()");
              boo = pdi.next_n(1,pdshold);

              panel.print(">     PropertyDefIterator.destroy()");
              pdi.destroy();
            }

            panel.print(">     FeatureType.destroy()");
            ftype.destroy();
            }

            panel.print(">     Feature.destroy()");
            fet.destroy();
            }

            panel.print(">     FeatureIterator.destroy()");
            itr.destroy();

            panel.print(">     destroy()");
            cfcRef.destroy();

        } catch (Exception e) {
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }
}
