import OGIS.*;
import OGIS.GeometryPackage.*;
import java.util.*;
import org.omg.CORBA.*;

/**
    Test Geometry Interfaces
 */

public class GeometryTest {


    private static WKSPoint wks_point = new WKSPoint(1.0,1.0);
    private static WKSPoint[] wks_points = {
        new WKSPoint(-1.0,-1.0),
        new WKSPoint( 1.0,-1.0),
        new WKSPoint( 1.0, 1.0),
        new WKSPoint(-1.0, 1.0)
    };
    private static WKSPoint[][] wks_points_2 = {
        {
            new WKSPoint(-0.5,-0.5),
            new WKSPoint(-0.2,-0.5),
            new WKSPoint(-0.2, 0.5),
            new WKSPoint(-0.5, 0.5)
        }, {
            new WKSPoint( 0.2,-0.5),
            new WKSPoint( 0.5,-0.5),
            new WKSPoint( 0.5, 0.5),
            new WKSPoint( 0.2, 0.5)
        }
    };
    private static WKSLinearPolygon wks_polygon =
        new WKSLinearPolygon( wks_points, wks_points_2 );
    private static WKSLinearPolygon[] wks_polygons = {
        wks_polygon, wks_polygon
    };

    private static EgenhoferElement[][] elements = {
        { EgenhoferElement.Empty, EgenhoferElement.NotEmpty,
                EgenhoferElement.NoTest},
        { EgenhoferElement.Empty, EgenhoferElement.NotEmpty,
                EgenhoferElement.NoTest},
        { EgenhoferElement.Empty, EgenhoferElement.NotEmpty,
                EgenhoferElement.NoTest}
    };
    private static EgenhoferOperator operator = 
        new EgenhoferOperator( elements );
        
    public static boolean testGeometry(
                    MainPanel        panel,
                    SpatialReferenceSystem srs,
                    GeometryFactory  gfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing GeometryFactory ..\n");

        try {

            ////////////////////////////////////////////////
            //  Point                                     //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [Point]");
            WKSGeometry wksg_p = new WKSGeometry();
            wksg_p.point( wks_point );
            Geometry g = gfRef.create_from_WKS( srs, wksg_p );

            panel.print(">     create() [Point]");
            Geometry g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [Point]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


            ////////////////////////////////////////////////
            //  LineString                                //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [LineString]");
            WKSGeometry wksg_ls = new WKSGeometry();
            wksg_ls.line_string( wks_points );
            g = gfRef.create_from_WKS( srs, wksg_ls );

            panel.print(">     create() [LineString]");
            g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [LineString]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


            ////////////////////////////////////////////////
            //  LinearPolygon                             //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [LinearPolygon]");
            WKSGeometry wksg_lp = new WKSGeometry();
            wksg_lp.linear_polygon( wks_polygon );
            g = gfRef.create_from_WKS( srs, wksg_lp );

            panel.print(">     create() [LinearPolygon]");
            g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [LinearPolygon]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


            ////////////////////////////////////////////////
            //  MultiPoint                                //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [MultiPoint]");
            WKSGeometry wksg_mp = new WKSGeometry();
            wksg_mp.multi_point( wks_points );
            g = gfRef.create_from_WKS( srs, wksg_mp );

            panel.print(">     create() [MultiPoint]");
            g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [MultiPoint]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


            ////////////////////////////////////////////////
            //  MultiLineString                           //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [MultiLineString]");
            WKSGeometry wksg_mls = new WKSGeometry();
            wksg_mls.multi_line_string( wks_points_2 );
            g = gfRef.create_from_WKS( srs, wksg_mls );

            panel.print(">     create() [MultiLineString]");
            g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [MultiLineString]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


            ////////////////////////////////////////////////
            //  MultiLinearPolygon                        //
            ////////////////////////////////////////////////
            panel.print(">     create_from_WKS() [MultiLinearPolygon]");
            WKSGeometry wksg_mlp = new WKSGeometry();
            wksg_mlp.multi_linear_polygon( wks_polygons );
            g = gfRef.create_from_WKS( srs, wksg_mlp );

            panel.print(">     create() [MultiLinearPolygon]");
            g2 = gfRef.create( g );

            if (testWKB){
                panel.print(">     create_from_WKB() [MultiLinearPolygon]");
                byte[] b = g.export_WKBGeometry();
                g2 = gfRef.create_from_WKB( srs, b );
            }


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }


    public static boolean testPoint(
                    MainPanel        panel,
                    SpatialReferenceSystem srs,
                    PointFactory     pfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing PointFactory & Point ..\n");

        try {
            panel.print(">     create_from_WKSPoint()");
            Point p = pfRef.create_from_WKSPoint( srs, wks_point );

            panel.print(">     create_from_Point()");
            Point p2 = pfRef.create_from_Point( p );

            // Geometry IF

            panel.print(">     copy()");
            Geometry g = p.copy();

            panel.print(">     dimension()");
            short dim = p.dimension();

            panel.print(">     range_envelope()");
            Envelope e = p.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = p.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = p.is_empty();

            panel.print(">     is_simple()");
            b = p.is_simple();

            panel.print(">     is_closed()");
            b = p.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = p.export();

            // Point IF

            panel.print(">     coordinates() [Get/Set]");
            WKSPoint wp = p.coordinates();
            p.coordinates( wks_point );

            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = p.export_WKBGeometry();

                panel.print(">     create_from_WKBPoint()");
                p2 = pfRef.create_from_WKBPoint( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = p.buffer(1.0);

                panel.print(">     convex_hull()");
                g = p.convex_hull();

                panel.print(">     boundary()");
                g = p.boundary();

                panel.print(">     equals()");
                b = p.equals(g);

                panel.print(">     touches()");
                b = p.touches(g);

                panel.print(">     contains()");
                b = p.contains(g);

                panel.print(">     within()");
                b = p.within(g);

                panel.print(">     disjoint()");
                b = p.disjoint(g);

                panel.print(">     crosses()");
                b = p.crosses(g);

                panel.print(">     overlaps()");
                b = p.overlaps(g);

                panel.print(">     intersects()");
                b = p.intersects(g);

                panel.print(">     relate()");
                b = p.relate(g, operator);

                panel.print(">     distance()");
                double d = p.distance(g);

                panel.print(">     intersection()");
                Geometry z = p.intersection(g);

                panel.print(">     union_op()");
                z = p.union_op(g);

                panel.print(">     difference()");
                z = p.difference(g);

                panel.print(">     symmetric_difference()");
                z = p.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            p.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }





    public static boolean testLineString(
                    MainPanel           panel,
                    SpatialReferenceSystem srs,
                    LineStringFactory  lsfRef,
                    boolean            testWKB,
                    boolean            testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing LineStringFactory & LineString ..\n");

        try {
            panel.print(">     create_from_WKSLineString()");
            LineString ls = lsfRef.create_from_WKSLineString( srs, wks_points );

            panel.print(">     create_from_LineString()");
            LineString ls2 = lsfRef.create_from_LineString( ls );

            // Geometry IF

            panel.print(">     copy()");
            Geometry g = ls.copy();

            panel.print(">     dimension()");
            short dim = ls.dimension();

            panel.print(">     range_envelope()");
            Envelope e = ls.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = ls.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = ls.is_empty();

            panel.print(">     is_simple()");
            b = ls.is_simple();

            panel.print(">     is_closed()");
            b = ls.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = ls.export();

            // Curve IF
            panel.print(">     length()");
            double d = ls.length();

            panel.print(">     start_point()");
            Point p = ls.start_point();

            panel.print(">     end_point()");
            p = ls.end_point();

            panel.print(">     start_point_as_WKS()");
            WKSPoint wp = ls.start_point_as_WKS();

            panel.print(">     end_point_as_WKS()");
            wp = ls.end_point_as_WKS();

            panel.print(">     is_planar()");
            b = ls.is_planar();

            panel.print(">     value()");
            p = ls.value(0.5);

            panel.print(">     value_as_WKS()");
            wp = ls.value_as_WKS(0.5);


            // LineString IF

            panel.print(">     num_points()");
            long n = ls.num_points();

            panel.print(">     get_point_by_index()");
            p = ls.get_point_by_index(0);

            panel.print(">     get_point_by_index_as_WKS()");
            wp = ls.get_point_by_index_as_WKS(0);

            panel.print(">     set_point_by_index()");
            ls.set_point_by_index(p, 0);

            panel.print(">     set_point_by_index_with_WKS()");
            ls.set_point_by_index_with_WKS(wp, 0);

            panel.print(">     insert_point_by_index()");
            ls.insert_point_by_index(p, 0);

            panel.print(">     insert_point_by_index_with_WKS()");
            ls.insert_point_by_index_with_WKS(wp, 0);

            panel.print(">     append_point()");
            ls.append_point(p);

            panel.print(">     append_point_with_WKS()");
            ls.append_point_with_WKS(wp);

            panel.print(">     delete_point_by_index()");
            ls.delete_point_by_index(0);


            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = ls.export_WKBGeometry();

                panel.print(">     create_from_WKBLineString()");
                ls2 = lsfRef.create_from_WKBLineString( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = ls.buffer(1.0);

                panel.print(">     convex_hull()");
                g = ls.convex_hull();

                panel.print(">     boundary()");
                g = ls.boundary();

                panel.print(">     equals()");
//                b = ls.equals(g);
                b = ((OGIS.Geometry)ls).equals(g);

                panel.print(">     touches()");
                b = ls.touches(g);

                panel.print(">     contains()");
                b = ls.contains(g);

                panel.print(">     within()");
                b = ls.within(g);

                panel.print(">     disjoint()");
                b = ls.disjoint(g);

                panel.print(">     crosses()");
                b = ls.crosses(g);

                panel.print(">     overlaps()");
                b = ls.overlaps(g);

                panel.print(">     intersects()");
                b = ls.intersects(g);

                panel.print(">     relate()");
                b = ls.relate(g, operator);

                panel.print(">     distance()");
                d = ls.distance(g);

                panel.print(">     intersection()");
                Geometry z = ls.intersection(g);

                panel.print(">     union_op()");
                z = ls.union_op(g);

                panel.print(">     difference()");
                z = ls.difference(g);

                panel.print(">     symmetric_difference()");
                z = ls.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            ls.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }





    public static boolean testLinearPolygon(
                    MainPanel        panel,
                    SpatialReferenceSystem  srs,
                    LinearPolygonFactory    lpfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing LinearPolygonFactory & LinearPolygon ..\n");

        try {
            panel.print(">     create_from_WKSLinearPolygon()");
            LinearPolygon lp = lpfRef.create_from_WKSLinearPolygon( srs, wks_polygon );

            panel.print(">     create_from_LinearPolygon()");
            LinearPolygon lp2 = lpfRef.create_from_LinearPolygon( lp );

            // Geometry IF

            panel.print(">     copy()");
            Geometry g = lp.copy();

            panel.print(">     dimension()");
            short dim = lp.dimension();

            panel.print(">     range_envelope()");
            Envelope e = lp.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = lp.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = lp.is_empty();

            panel.print(">     is_simple()");
            b = lp.is_simple();

            panel.print(">     is_closed()");
            b = lp.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = lp.export();

            // Surface IF

            panel.print(">     area()");
            double d = lp.area();

            panel.print(">     centroid()");
            Point p = lp.centroid();

            panel.print(">     centroid_as_WKS()");
            WKSPoint wp = lp.centroid_as_WKS();

            panel.print(">     point_on_surface()");
            p = lp.point_on_surface();

            panel.print(">     point_on_surface_as_WKS()");
            wp = lp.point_on_surface_as_WKS();

            panel.print(">     is_planar()");
            b = lp.is_planar();

            // Polygon IF

            panel.print(">     exterior_ring()");
            Ring r = lp.exterior_ring();

            panel.print(">     exterior_ring_as_WKS()");
            wg = lp.exterior_ring_as_WKS();

            panel.print(">     interior_rings()");
            MultiCurve mc = lp.interior_rings();

            panel.print(">     interior_rings_as_WKS()");
            wg = lp.interior_rings_as_WKS();


            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = lp.export_WKBGeometry();

                panel.print(">     create_from_WKBLinearPolygon()");
                lp2 = lpfRef.create_from_WKBLinearPolygon( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = lp.buffer(1.0);

                panel.print(">     convex_hull()");
                g = lp.convex_hull();

                panel.print(">     boundary()");
                g = lp.boundary();

                panel.print(">     equals()");
//                b = lp.equals(g);
                b = ((OGIS.Geometry)lp).equals(g);

                panel.print(">     touches()");
                b = lp.touches(g);

                panel.print(">     contains()");
                b = lp.contains(g);

                panel.print(">     within()");
                b = lp.within(g);

                panel.print(">     disjoint()");
                b = lp.disjoint(g);

                panel.print(">     crosses()");
                b = lp.crosses(g);

                panel.print(">     overlaps()");
                b = lp.overlaps(g);

                panel.print(">     intersects()");
                b = lp.intersects(g);

                panel.print(">     relate()");
                b = lp.relate(g, operator);

                panel.print(">     distance()");
                d = lp.distance(g);

                panel.print(">     intersection()");
                Geometry z = lp.intersection(g);

                panel.print(">     union_op()");
                z = lp.union_op(g);

                panel.print(">     difference()");
                z = lp.difference(g);

                panel.print(">     symmetric_difference()");
                z = lp.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            lp.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }





    public static boolean testMultiPoint(
                    MainPanel        panel,
                    SpatialReferenceSystem srs,
                    MultiPointFactory  mpfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing MultiPointFactory & MultiPoint ..\n");

        try {
            panel.print(">     create_from_WKSMultiPoint()");
            MultiPoint mp = mpfRef.create_from_WKSMultiPoint( srs, wks_points );

            panel.print(">     create_from_MultiPoint()");
            MultiPoint mp2 = mpfRef.create_from_MultiPoint( mp );

            // GeometryIF

            panel.print(">     copy()");
            Geometry g = mp.copy();

            panel.print(">     dimension()");
            short dim = mp.dimension();

            panel.print(">     range_envelope()");
            Envelope e = mp.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = mp.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = mp.is_empty();

            panel.print(">     is_simple()");
            b = mp.is_simple();

            panel.print(">     is_closed()");
            b = mp.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = mp.export();

            // GeometryCollection IF

            panel.print(">     number_elements()");
            long n = mp.number_elements();

            panel.print(">     create_iterator()");
            GeometryIterator itr = mp.create_iterator();

            panel.print(">     add_element()");
            panel.print(">     GeometryIterator.more()");
            panel.print(">     GeometryIterator.next()");
            while ( itr.more() ) {
                g = itr.next();
                mp2.add_element(g);
            }

            panel.print(">     merge()");
            mp.merge(mp2);

            panel.print(">     remove_all_elements()");
            mp2.remove_all_elements();

            panel.print(">     GeometryIterator.reset()");
            itr.reset();

            if ( itr.more() ) {

                itr.next();

                panel.print(">     retrieve_element_at()");
                g = mp.retrieve_element_at(itr);

                panel.print(">     replace_element_at()");
                mp.replace_element_at(g, itr);

                panel.print(">     remove_element_at()");
                mp.remove_element_at(itr);

                panel.print(">     insert_element_at()");
                mp.insert_element_at(g,itr);
            }

            panel.print(">     GeometryIterator.destroy()");
            itr.destroy();


            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = mp.export_WKBGeometry();

                panel.print(">     create_from_WKBMultiPoint()");
                mp2 = mpfRef.create_from_WKBMultiPoint( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = mp.buffer(1.0);

                panel.print(">     convex_hull()");
                g = mp.convex_hull();

                panel.print(">     boundary()");
                g = mp.boundary();

                panel.print(">     equals()");
//                b = mp.equals(g);
                b = ((OGIS.Geometry)mp).equals(g);

                panel.print(">     touches()");
                b = mp.touches(g);

                panel.print(">     contains()");
                b = mp.contains(g);

                panel.print(">     within()");
                b = mp.within(g);

                panel.print(">     disjoint()");
                b = mp.disjoint(g);

                panel.print(">     crosses()");
                b = mp.crosses(g);

                panel.print(">     overlaps()");
                b = mp.overlaps(g);

                panel.print(">     intersects()");
                b = mp.intersects(g);

                panel.print(">     relate()");
                b = mp.relate(g, operator);

                panel.print(">     distance()");
                double d = mp.distance(g);

                panel.print(">     intersection()");
                Geometry z = mp.intersection(g);

                panel.print(">     union_op()");
                z = mp.union_op(g);

                panel.print(">     difference()");
                z = mp.difference(g);

                panel.print(">     symmetric_difference()");
                z = mp.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            mp.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }





    public static boolean testMultiLineString(
                    MainPanel        panel,
                    SpatialReferenceSystem srs,
                    MultiLineStringFactory  mlsfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing MultiLineStringFactory & MultiLineString ..\n");

        try {
            panel.print(">     create_from_WKSMultiLineString()");
            MultiLineString mls =
                mlsfRef.create_from_WKSMultiLineString( srs, wks_points_2 );

            panel.print(">     create_from_MultiLineString()");
            MultiLineString mls2 = mlsfRef.create_from_MultiLineString( mls );

            // Geometry IF

            panel.print(">     copy()");
            Geometry g = mls.copy();

            panel.print(">     dimension()");
            short dim = mls.dimension();

            panel.print(">     range_envelope()");
            Envelope e = mls.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = mls.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = mls.is_empty();

            panel.print(">     is_simple()");
            b = mls.is_simple();

            panel.print(">     is_closed()");
            b = mls.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = mls.export();

            // GeometryCollection IF

            panel.print(">     number_elements()");
            long n = mls.number_elements();

            panel.print(">     create_iterator()");
            GeometryIterator itr = mls.create_iterator();

            panel.print(">     add_element()");
            panel.print(">     GeometryIterator.more()");
            panel.print(">     GeometryIterator.next()");
            while ( itr.more() ) {
                g = itr.next();
                mls2.add_element(g);
            }

            panel.print(">     merge()");
            mls.merge(mls2);

            panel.print(">     remove_all_elements()");
            mls2.remove_all_elements();

            panel.print(">     GeometryIterator.reset()");
            itr.reset();

            if ( itr.more() ) {

                itr.next();

                panel.print(">     retrieve_element_at()");
                g = mls.retrieve_element_at(itr);

                panel.print(">     replace_element_at()");
                mls.replace_element_at(g, itr);

                panel.print(">     remove_element_at()");
                mls.remove_element_at(itr);

                panel.print(">     insert_element_at()");
                mls.insert_element_at(g,itr);
            }

            panel.print(">     GeometryIterator.destroy()");
            itr.destroy();

            // MultiLineString IF

            double d = mls.length();



            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = mls.export_WKBGeometry();

                panel.print(">     create_from_WKBMultiLineString()");
                mls2 = mlsfRef.create_from_WKBMultiLineString( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = mls.buffer(1.0);

                panel.print(">     convex_hull()");
                g = mls.convex_hull();

                panel.print(">     boundary()");
                g = mls.boundary();

                panel.print(">     equals()");
//                b = mls.equals(g);
                b = ((OGIS.Geometry)mls).equals(g);

                panel.print(">     touches()");
                b = mls.touches(g);

                panel.print(">     contains()");
                b = mls.contains(g);

                panel.print(">     within()");
                b = mls.within(g);

                panel.print(">     disjoint()");
                b = mls.disjoint(g);

                panel.print(">     crosses()");
                b = mls.crosses(g);

                panel.print(">     overlaps()");
                b = mls.overlaps(g);

                panel.print(">     intersects()");
                b = mls.intersects(g);

                panel.print(">     relate()");
                b = mls.relate(g, operator);

                panel.print(">     distance()");
                d = mls.distance(g);

                panel.print(">     intersection()");
                Geometry z = mls.intersection(g);

                panel.print(">     union_op()");
                z = mls.union_op(g);

                panel.print(">     difference()");
                z = mls.difference(g);

                panel.print(">     symmetric_difference()");
                z = mls.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            mls.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }





    public static boolean testMultiLinearPolygon(
                    MainPanel        panel,
                    SpatialReferenceSystem srs,
                    MultiLinearPolygonFactory  mlpfRef,
                    boolean          testWKB,
                    boolean          testSpatialOp
                    )
    {
        boolean result = true;
        panel.print("\n> testing MultiLinearPolygonFactory & MultiLinearPolygon ..\n");

        try {
            panel.print(">     create_from_WKSMultiLinearPolygon()");
            MultiLinearPolygon mlp =
                mlpfRef.create_from_WKSMultiLinearPolygon( srs, wks_polygons );

            panel.print(">     create_from_MultiLinearPolygon()");
            MultiLinearPolygon mlp2 =
                mlpfRef.create_from_MultiLinearPolygon( mlp );

            // Geometry IF

            panel.print(">     copy()");
            Geometry g = mlp.copy();

            panel.print(">     dimension()");
            short dim = mlp.dimension();

            panel.print(">     range_envelope()");
            Envelope e = mlp.range_envelope();

            panel.print(">     spatial_reference_system()");
            SpatialReferenceSystem sys = mlp.spatial_reference_system();

            panel.print(">     is_empty()");
            boolean  b = mlp.is_empty();

            panel.print(">     is_simple()");
            b = mlp.is_simple();

            panel.print(">     is_closed()");
            b = mlp.is_closed();

            panel.print(">     export()");
            WKSGeometry wg = mlp.export();

            // GeometryCollectionIF

            panel.print(">     number_elements()");
            long n = mlp.number_elements();

            panel.print(">     create_iterator()");
            GeometryIterator itr = mlp.create_iterator();

            panel.print(">     add_element()");
            panel.print(">     GeometryIterator.more()");
            panel.print(">     GeometryIterator.next()");
            while ( itr.more() ) {
                g = itr.next();
                mlp2.add_element(g);
            }

            panel.print(">     merge()");
            mlp.merge(mlp2);

            panel.print(">     remove_all_elements()");
            mlp2.remove_all_elements();

            panel.print(">     GeometryIterator.reset()");
            itr.reset();

            if ( itr.more() ) {

                itr.next();

                panel.print(">     retrieve_element_at()");
                g = mlp.retrieve_element_at(itr);

                panel.print(">     replace_element_at()");
                mlp.replace_element_at(g, itr);

                panel.print(">     remove_element_at()");
                mlp.remove_element_at(itr);

                panel.print(">     insert_element_at()");
                mlp.insert_element_at(g,itr);
            }

            panel.print(">     GeometryIterator.destroy()");
            itr.destroy();

            // MultiSurface IF

            double d = mlp.area();



            if (testWKB){
                panel.print(">     export_WKBGeometry()");
                byte[] bytes = mlp.export_WKBGeometry();

                panel.print(">     create_from_WKBMultiLinearPolygon()");
                mlp2 = mlpfRef.create_from_WKBMultiLinearPolygon( srs, bytes );
            }

            if (testSpatialOp){
                panel.print(">     buffer()");
                g = mlp.buffer(1.0);

                panel.print(">     convex_hull()");
                g = mlp.convex_hull();

                panel.print(">     boundary()");
                g = mlp.boundary();

                panel.print(">     equals()");
//                b = mlp.equals(g);
                b = ((OGIS.Geometry)mlp).equals(g);

                panel.print(">     touches()");
                b = mlp.touches(g);

                panel.print(">     contains()");
                b = mlp.contains(g);

                panel.print(">     within()");
                b = mlp.within(g);

                panel.print(">     disjoint()");
                b = mlp.disjoint(g);

                panel.print(">     crosses()");
                b = mlp.crosses(g);

                panel.print(">     overlaps()");
                b = mlp.overlaps(g);

                panel.print(">     intersects()");
                b = mlp.intersects(g);

                panel.print(">     relate()");
                b = mlp.relate(g, operator);

                panel.print(">     distance()");
                d = mlp.distance(g);

                panel.print(">     intersection()");
                Geometry z = mlp.intersection(g);

                panel.print(">     union_op()");
                z = mlp.union_op(g);

                panel.print(">     difference()");
                z = mlp.difference(g);

                panel.print(">     symmetric_difference()");
                z = mlp.symmetric_difference(g);
            }

            panel.print(">     destroy()");
            mlp.destroy();


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }

        return result;
    }


}