import OGIS.*;
import java.util.*;
import java.awt.event.*;
import org.omg.CORBA.*;


/**
    OGC CORBA Spec.  (for VisiBroker)
 */

public class OGCTestSuite {

    
    // Common
    private MainPanel   panel;
    private Properties  prop;
    private ORB         orb;

    // References of SRS Servers
    private SpatialReferenceSystemFactory     srsfRef;
    private SpatialReferenceComponentFactory  srcfRef;
    private SpatialReferenceSystem            srs; // create from WKT

    // References of Feature Servers
    private ContainerFeatureCollection                  cfcRef;
    private ContainerFeatureCollectionFactory           cfcfRef;
    private QueryableContainerFeatureCollection         qcfcRef;
    private QueryableContainerFeatureCollectionFactory  qcfcfRef;
    private FeatureCollectionFactory                    fcfRef;
    
    // References of FeatureType Servers
    private FeatureTypeFactory  ftfRef;
    
    // References of Geometry Servers
    private GeometryFactory            gfRef;
    private PointFactory               pfRef;
    private LineStringFactory          lsfRef;
    private LinearPolygonFactory       lpfRef;
    private MultiPointFactory          mpfRef;
    private MultiLineStringFactory     mlsfRef;
    private MultiLinearPolygonFactory  mlpfRef;
    
    /**
        Constructor
     */
    OGCTestSuite( String propName ){

        prop = new Properties();
        
        if ( propName != null ){
            try {
                prop.load( this.getClass().getResourceAsStream( propName ) );
            } catch (Exception e){
                e.printStackTrace();
            }
        }

        // Create Setting Panel
        panel = new MainPanel( this, prop );
        panel.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                System.exit(0);
            }
        } );

        
    }


    /**
        Set Referential Object for ORB
     */
    public void initORB( String hostName, int portNum ) {

        panel.print( "> initializing ORB connection to " + 
                     hostName + ":" + portNum );

        try {
    
            Properties orb_prop = new Properties();
            orb_prop.put( "org.omg.CORBA.ORBInitialHost", hostName );
            orb_prop.put( "org.omg.CORBA.ORBInitialPort",
                                      String.valueOf( portNum ) );
            orb = ORB.init( (String[])null, orb_prop );
            
        } catch( Exception e ) {
            panel.print("> ERROR: Init ORB Failed!!");
            e.printStackTrace();
        }
        
        panel.print("> OK");
    }


    /**
        Bind SpatialReferenceSystemFactory
     */
    public void bindSRSFactory( 
                    String srsfName   // SpatialReferenceSystemFactory
                    )
    {
        panel.print(">> binding " + srsfName);
    
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            srsfRef = SpatialReferenceSystemFactoryHelper.bind( orb, srsfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }


    /**
        Bind SpatialReferenceComponentFactory
     */
    public void bindSRCFactory( 
                    String srcfName   // SpatialReferenceComponentFactory
                    )
    {
        panel.print(">> binding " + srcfName);
    
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            srcfRef = SpatialReferenceComponentFactoryHelper.bind( orb, srcfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }


    /**
        Run SpatialReferenceSystem Test
     */
    public void testSRSServer( String wkt )
    {
        srs = SRSTest.testSRS( panel, srsfRef, wkt );

        if ( srs != null ){ // failure
            panel.print("> succeeded ! ");
        }
    }


    /**
        Run SpatialReferenceComponent Test
     */
    public void testSRCServer( boolean testGeoTrans )  // GeometricTransform
    {
        boolean result = SRCTest.testSRC( 
                            panel, srcfRef, testGeoTrans );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Bind QueryableContainerFeatureCollectionFactory
     */
    public void bindQCFCFServer( String qcfcfName, String fcfName )
    {
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            panel.print("> binding " + qcfcfName);
            qcfcfRef = QueryableContainerFeatureCollectionFactoryHelper.bind( 
                        orb, qcfcfName );

            panel.print("> binding " + fcfName);
            fcfRef = FeatureCollectionFactoryHelper.bind( orb, fcfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }

    /**
        Bind QueryableContainerFeatureCollection
     */
    public void bindQCFCServer( String qcfcName, String fcfName )
    {
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            panel.print("> binding " + qcfcName);
            qcfcRef = QueryableContainerFeatureCollectionHelper.bind( 
                        orb, qcfcName );

            panel.print("> binding " + fcfName);
            fcfRef = FeatureCollectionFactoryHelper.bind( orb, fcfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }

    /**
        Bind ContainerFeatureCollectionFactory
     */
    public void bindCFCFServer( String cfcfName, String fcfName )
    {
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            panel.print("> binding " + cfcfName);
            cfcfRef = ContainerFeatureCollectionFactoryHelper.bind( 
                        orb, cfcfName );

            panel.print("> binding " + fcfName);

            fcfRef = FeatureCollectionFactoryHelper.bind( orb, fcfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }

    /**
        Bind ContainerFeatureCollection
     */
    public void bindCFCServer( String cfcName, String fcfName )
    {
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/
            panel.print("> binding " + cfcName);            
            cfcRef = ContainerFeatureCollectionHelper.bind( orb, cfcName );

            panel.print("> binding " + fcfName);

            fcfRef = FeatureCollectionFactoryHelper.bind( orb, fcfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }



    /**
        Run QueryableContainerFeatureCollectionFactory Test
     */
    public void testQCFCFServer( int revision, String ftName, String fcftName )
    {
        boolean result = 
            FeatureTest.testQCFCF( orb, panel, qcfcfRef, ftName, revision, fcfRef, fcftName );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Run QueryableContainerFeatureCollection Test
     */
    public void testQCFCServer( int revision, String fcftName )
    {
        boolean result = 
            FeatureTest.testQCFC( orb, panel, qcfcRef, revision, fcfRef, fcftName );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Run ContainerFeatureCollectionFactory Test
     */
    public void testCFCFServer( int revision, String ftName, String fcftName )
    {
        boolean result = 
            FeatureTest.testCFCF( orb, panel, cfcfRef, ftName, revision, fcfRef, fcftName );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Run ContainerFeatureCollection Test
     */
    public void testCFCServer( int revision, String fcftName )
    {
        boolean result = 
            FeatureTest.testCFC( orb, panel, cfcRef, revision, fcfRef, fcftName );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Bind FeatureTypeFactory
     */
    public void bindFTFServer( String ftfName )
    {
        try {
            panel.print("> binding " + ftfName);
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/            
            ftfRef = FeatureTypeFactoryHelper.bind( orb, ftfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/
            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }



    /**
        Run FeatureTypeFactory Test
     */
    public void testFTFServer()
    {
        boolean result = 
            FeatureTypeTest.testFTF( panel, ftfRef );

        if (result){
            panel.print("> succeeded ! ");
        }
    }


    /**
        Bind Geometry Servers
     */
    public void bindGeometryServers( 
                    String gfName,   // GeometryFactory
                    String pfName,   // Point..
                    String lsfName,  // LineString..
                    String lpfName,  // LinearPolygon..
                    String mpfName,  // MultiPoint..
                    String mlsfName, // MultiLineString..
                    String mlpfName  // MultiLinearPolygon..
                    )
    {   
        panel.print("\n>> binding Geometry servers ..\n");
    
        try {
/******************************************************************************
 *   !#@ ADAPTATION  BEGIN
 * This adaptation modifies ....
 ******************************************************************************/            
            // GeometryFactory
            panel.print("> binding " + gfName);
            gfRef = GeometryFactoryHelper.bind( orb, gfName );

            // PointFactory
            panel.print("> binding " + pfName);
            pfRef = PointFactoryHelper.bind( orb, pfName );

            // LineStringFactory
            panel.print("> binding " + lsfName);
            lsfRef = LineStringFactoryHelper.bind( orb, lsfName );

            // LinearPolygonFactory
            panel.print("> binding " + lpfName);
            lpfRef = LinearPolygonFactoryHelper.bind( orb, lpfName );

            // MultiPointFactory
            panel.print("> binding " + mpfName);
            mpfRef = MultiPointFactoryHelper.bind( orb, mpfName );

            // MultiLineStringFactory
            panel.print("> binding " + mlsfName);
            mlsfRef = MultiLineStringFactoryHelper.bind( orb, mlsfName );

            // MultiLinearPolygonFactory
            panel.print("> binding " + mlpfName);
            mlpfRef = MultiLinearPolygonFactoryHelper.bind( orb, mlpfName );
/******************************************************************************
 *   !#@ ADAPTATION END
 ******************************************************************************/


            panel.print("> succeeded ! ");

        } catch (Exception e){
            panel.print("> ERROR !");
            e.printStackTrace();
        }
    }
    

    /**
        Run Geometry Test
     */
    public void testGeometryServers( 
                    boolean testWKB,
                    boolean testSpatialOp
                    )
    {   
        boolean result = true;

        if (result){
            result = GeometryTest.testGeometry( 
                        panel, srs, gfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testPoint( 
                        panel, srs, pfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testLineString( 
                        panel, srs, lsfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testLinearPolygon( 
                        panel, srs, lpfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testMultiPoint( 
                        panel, srs, mpfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testMultiLineString( 
                        panel, srs, mlsfRef, testWKB, testSpatialOp );
        }

        if (result){
            result = GeometryTest.testMultiLinearPolygon( 
                        panel, srs, mlpfRef, testWKB, testSpatialOp );
        }

        if (result){
            panel.print("> succeeded ! ");
        }
    }



    /**
        Main method
     */
    public static void main( String[] argv ){

        String propName = null;

        if ( argv.length == 0 ){
            System.out.println("\nusage: java OGCTestSuite <PropertyFile>\n");
        } else {
            propName = argv[0];
        }

        // Create TestSuite Object
        OGCTestSuite testObj = new OGCTestSuite( propName );
  
    }

}