import OGIS.*;
import java.util.*;
import org.omg.CORBA.*;

/**
    Test SRC
 */

public class SRCTest {

    // EPSG Code
    private static int 
        pcs_code = 26910, // ProjectedCoordinateSystem
        gcs_code = 4269,  // GeographicCoordinateSystem
        prj_code = 16010, // TranverseMercator
        gt_code = 8070,      // GeographicTransform
        hd_code = 6269,   // HorizontalDatum
        el_code = 7019,   // Ellipsoid
        pm_code = 8901,   // PrimeMeridian
        lu_code = 9001,   // LinearUnit
        au_code = 9110    // AngularUnit
        ;

    private static WKSPoint wks_point = new WKSPoint(1.0,1.0);


    public static boolean testSRC(
                    MainPanel                         panel,
                    SpatialReferenceComponentFactory  srcfRef,
                    boolean                           testGeoTrans
                    )
    {
        boolean result = true;
        panel.print("\n> testing SpatialReferenceComponentFactory ..\n");

        try {

            // (1)
            panel.print(">     create_projected_coordinate_system()");
            ProjectedCoordinateSystem pcs = 
                srcfRef.create_projected_coordinate_system(pcs_code);
            if ( testPCS(panel,pcs) == false ){
                return false;
            }
            panel.print(">     ProjectedCoordinateSystem  OK \n");

            // (2)
            panel.print(">     create_geographic_coordinate_system()");
            GeographicCoordinateSystem gcs = 
                srcfRef.create_geographic_coordinate_system(gcs_code);
            if ( testGCS(panel,gcs) == false ){
                return false;
            }
            panel.print(">     GeographicCoordinateSystem  OK \n");

            // (3)
            panel.print(">     create_projection()");
            Projection prj = srcfRef.create_projection(prj_code);
            if ( testPRJ(panel,prj) == false ){
                return false;
            }
            panel.print(">     Projection  OK \n");

            // (4)
            panel.print(">     create_horizontal_datum()");
            HorizontalDatum hd = srcfRef.create_horizontal_datum(hd_code);
            if ( testHD(panel,hd) == false ){
                return false;
            }
            panel.print(">     HorizontalDatum  OK \n");

            // (5)
            panel.print(">     create_ellipsoid()");
            Ellipsoid el = srcfRef.create_ellipsoid(el_code);
            if ( testEL(panel,el) == false ){
                return false;
            }
            panel.print(">     Ellipsoid  OK \n");

            // (6)
            panel.print(">     create_prime_meridian()");
            PrimeMeridian pm = srcfRef.create_prime_meridian(pm_code);
            if ( testPM(panel,pm) == false ){
                return false;
            }
            panel.print(">     PrimeMeridian  OK \n");

            // (7)
            panel.print(">     create_linear_unit()");
            LinearUnit lu = srcfRef.create_linear_unit(lu_code);
            if ( testLU(panel,lu) == false ){
                return false;
            }
            panel.print(">     LinearUnit  OK \n");

            // (8)
            panel.print(">     create_angular_unit()");
            AngularUnit au = srcfRef.create_angular_unit(au_code);
            if ( testAU(panel,au) == false ){
                return false;
            }
            panel.print(">     AngularUnit  OK \n");


            // (9)
            if (testGeoTrans){

                panel.print(">     create_geographic_transform()");
                GeographicTransform gt = 
                    srcfRef.create_geographic_transform(gt_code);

                if ( testGT(panel,gt) == false ){
                    return false;
                }
                panel.print(">     GeographicTramsform  OK \n");
            }


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }
    


    
    
    public static boolean testPCS(
                    MainPanel                  panel,
                    ProjectedCoordinateSystem  pcs
                    )
    {
        boolean result = true;
        panel.print("\n> testing ProjectedCoordinateSystem ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = pcs.name();
            pcs.name(s);

            panel.print(">     authority() [Get/Set]");
            s = pcs.authority();
            pcs.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = pcs.code();
            pcs.code(i);

            panel.print(">     alias() [Get/Set]");
            s = pcs.alias();
            pcs.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = pcs.abbreviation();
            pcs.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = pcs.remarks();
            pcs.remarks(s);

            panel.print(">     well_known_text()");
            s = pcs.well_known_text();

            // ProjectedCoordinateSystem IF

            panel.print(">     usage() [Get/Set]");
            s = pcs.usage();
            pcs.usage(s);
            
            panel.print(">     geographic_coordinate_system()");
            GeographicCoordinateSystem gcs = 
                pcs.geographic_coordinate_system();
            pcs.geographic_coordinate_system(gcs);

            panel.print(">     linear_units() [Get/Set]");
            LinearUnit lu = pcs.linear_units();
            pcs.linear_units(lu);

            panel.print(">     base_projection() [Get/Set]");
            Projection prj = pcs.base_projection();
            pcs.base_projection(prj);

            panel.print(">     parameters()");
            ParameterList plist = pcs.parameters();

            panel.print(">     forward()");
            WKSGeometry wksg = new WKSGeometry();
            wksg.point( wks_point );
            WKSGeometry wg = pcs.forward( wksg );

            panel.print(">     inverse()");
            WKSGeometry wg2 = pcs.inverse(wg);

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testGCS(
                    MainPanel                  panel,
                    GeographicCoordinateSystem  gcs
                    )
    {
        boolean result = true;
        panel.print("\n> testing GeographicCoordinateSystem ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = gcs.name();
            gcs.name(s);

            panel.print(">     authority() [Get/Set]");
            s = gcs.authority();
            gcs.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = gcs.code();
            gcs.code(i);

            panel.print(">     alias() [Get/Set]");
            s = gcs.alias();
            gcs.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = gcs.abbreviation();
            gcs.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = gcs.remarks();
            gcs.remarks(s);

            panel.print(">     well_known_text()");
            s = gcs.well_known_text();

            // GeographicCoordinateSystem IF

            panel.print(">     usage() [Get/Set]");
            s = gcs.usage();
            gcs.usage(s);
            
            panel.print(">     horizontal_datum() [Get/Set]");
            HorizontalDatum hd = gcs.horizontal_datum();
            gcs.horizontal_datum(hd);

            panel.print(">     angular_unit() [Get/Set]");
            AngularUnit au = gcs.angular_unit();
            gcs.angular_unit(au);

            panel.print(">     prime_meridian() [Get/Set]");
            PrimeMeridian pm = gcs.prime_meridian();
            gcs.prime_meridian(pm);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testHD(
                    MainPanel        panel,
                    HorizontalDatum  hd
                    )
    {
        boolean result = true;
        panel.print("\n> testing HorizontalDatum ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = hd.name();
            hd.name(s);

            panel.print(">     authority() [Get/Set]");
            s = hd.authority();
            hd.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = hd.code();
            hd.code(i);

            panel.print(">     alias() [Get/Set]");
            s = hd.alias();
            hd.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = hd.abbreviation();
            hd.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = hd.remarks();
            hd.remarks(s);

            panel.print(">     well_known_text()");
            s = hd.well_known_text();

            // HorizontalDatum IF

            panel.print(">     base_ellipsoid() [Get/Set]");
            Ellipsoid el = hd.base_ellipsoid();
            hd.base_ellipsoid(el);
            

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testEL(
                    MainPanel  panel,
                    Ellipsoid  el
                    )
    {
        boolean result = true;
        panel.print("\n> testing Ellipsoid ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = el.name();
            el.name(s);

            panel.print(">     authority() [Get/Set]");
            s = el.authority();
            el.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = el.code();
            el.code(i);

            panel.print(">     alias() [Get/Set]");
            s = el.alias();
            el.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = el.abbreviation();
            el.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = el.remarks();
            el.remarks(s);

            panel.print(">     well_known_text()");
            s = el.well_known_text();

            // Ellipsoid IF

            panel.print(">     semi_major_axis() [Get/Set]");
            double d = el.semi_major_axis();
            el.semi_major_axis(d);

            panel.print(">     semi_minor_axis() [Get/Set]");
            d = el.semi_minor_axis();
            el.semi_minor_axis(d);

            panel.print(">     inverse_flattening() [Get/Set]");
            d = el.inverse_flattening();
            el.inverse_flattening(d);

            panel.print(">     axis_unit() [Get/Set]");
            LinearUnit lu = el.axis_unit();
            el.axis_unit(lu);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testLU(
                    MainPanel   panel,
                    LinearUnit  lu
                    )
    {
        boolean result = true;
        panel.print("\n> testing LinearUnit ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = lu.name();
            lu.name(s);

            panel.print(">     authority() [Get/Set]");
            s = lu.authority();
            lu.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = lu.code();
            lu.code(i);

            panel.print(">     alias() [Get/Set]");
            s = lu.alias();
            lu.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = lu.abbreviation();
            lu.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = lu.remarks();
            lu.remarks(s);

            panel.print(">     well_known_text()");
            s = lu.well_known_text();

            // LinearUnit IF

            panel.print(">     metres_per_unit() [Get/Set]");
            double d = lu.metres_per_unit();
            lu.metres_per_unit(d);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testAU(
                    MainPanel    panel,
                    AngularUnit  au
                    )
    {
        boolean result = true;
        panel.print("\n> testing AngularUnit ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = au.name();
            au.name(s);

            panel.print(">     authority() [Get/Set]");
            s = au.authority();
            au.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = au.code();
            au.code(i);

            panel.print(">     alias() [Get/Set]");
            s = au.alias();
            au.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = au.abbreviation();
            au.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = au.remarks();
            au.remarks(s);

            panel.print(">     well_known_text()");
            s = au.well_known_text();

            // AngularUnit IF

            panel.print(">     radians_per_unit() [Get/Set]");
            double d = au.radians_per_unit();
            au.radians_per_unit(d);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }






   public static boolean testPM(
                    MainPanel    panel,
                    PrimeMeridian  pm
                    )
    {
        boolean result = true;
        panel.print("\n> testing PrimeMeridian ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = pm.name();
            pm.name(s);

            panel.print(">     authority() [Get/Set]");
            s = pm.authority();
            pm.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = pm.code();
            pm.code(i);

            panel.print(">     alias() [Get/Set]");
            s = pm.alias();
            pm.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = pm.abbreviation();
            pm.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = pm.remarks();
            pm.remarks(s);

            panel.print(">     well_known_text()");
            s = pm.well_known_text();

            // PrimeMeridian IF

            panel.print(">     longitude() [Get/Set]");
            double d = pm.longitude();
            pm.longitude(d);

            panel.print(">     angular_units() [Get/Set]");
            AngularUnit au = pm.angular_units();
            pm.angular_units(au);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testPRJ(
                    MainPanel    panel,
                    Projection   prj
                    )
    {
        boolean result = true;
        panel.print("\n> testing Projection ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = prj.name();
            prj.name(s);

            panel.print(">     authority() [Get/Set]");
            s = prj.authority();
            prj.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = prj.code();
            prj.code(i);

            panel.print(">     alias() [Get/Set]");
            s = prj.alias();
            prj.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = prj.abbreviation();
            prj.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = prj.remarks();
            prj.remarks(s);

            panel.print(">     well_known_text()");
            s = prj.well_known_text();

            // Projection IF

            panel.print(">     usage()");
            s = prj.usage();

            panel.print(">     classification()");
            s = prj.classification();

            panel.print(">     forward()");
            WKSGeometry wksg = new WKSGeometry();
            wksg.point( wks_point );
            WKSGeometry wg = prj.forward( wksg );

            panel.print(">     inverse()");
            WKSGeometry wg2 = prj.inverse(wg);

            panel.print(">     parameters()");
            ParameterList plist = prj.parameters();
            
            if ( testPList(panel,plist) == false ){
                return false;
            }
            panel.print(">     ParameterList  OK \n");

            panel.print(">     angular_units() [Get/Set]");
            AngularUnit au = prj.angular_units();
            prj.angular_units(au);

            panel.print(">     linear_units() [Get/Set]");
            LinearUnit lu = prj.linear_units();
            prj.linear_units(lu);

            panel.print(">     base_ellipsoid() [Get/Set]");
            Ellipsoid el = prj.base_ellipsoid();
            prj.base_ellipsoid(el);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }





   public static boolean testPList(
                    MainPanel      panel,
                    ParameterList  plist
                    )
    {
        boolean result = true;
        panel.print("\n> testing ParameterList ..\n");

        try {

            // ParameterList IF

            panel.print(">     number_parameters() [Get/Set]");
            int i = plist.number_parameters();

            panel.print(">     get_default_parameters()");
            Parameter[] params = plist.get_default_parameters();

            panel.print(">     get_parameters()");
            params = plist.get_parameters();

            panel.print(">     set_parameters()");
            plist.set_parameters(params);

            if ( testParam(panel,params[0]) == false ){
                return false;
            }
            panel.print(">     Parameter  OK \n");
            

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }






   public static boolean testParam(
                    MainPanel      panel,
                    Parameter      param
                    )
    {
        boolean result = true;
        panel.print("\n> testing Parameter ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = param.name();
            param.name(s);

            panel.print(">     authority() [Get/Set]");
            s = param.authority();
            param.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = param.code();
            param.code(i);

            panel.print(">     alias() [Get/Set]");
            s = param.alias();
            param.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = param.abbreviation();
            param.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = param.remarks();
            param.remarks(s);

            panel.print(">     well_known_text()");
            s = param.well_known_text();

            // Parameter IF

            panel.print(">     units() [Get/Set]");
            Unit u = param.units();
            param.units(u);

            panel.print(">     value()");
            double d = param.value();
            param.value(d);


        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }






   public static boolean testGT(
                    MainPanel            panel,
                    GeographicTransform  gt
                    )
    {
        boolean result = true;
        panel.print("\n> testing GeographicTransform ..\n");

        try {

            // SpatialReferenceInfo IF

            panel.print(">     name() [Get/Set]");
            String s = gt.name();
            gt.name(s);

            panel.print(">     authority() [Get/Set]");
            s = gt.authority();
            gt.authority(s);

            panel.print(">     code() [Get/Set]");
            int i = gt.code();
            gt.code(i);

            panel.print(">     alias() [Get/Set]");
            s = gt.alias();
            gt.alias(s);

            panel.print(">     abbreviation() [Get/Set]");
            s = gt.abbreviation();
            gt.abbreviation(s);

            panel.print(">     remarks() [Get/Set]");
            s = gt.remarks();
            gt.remarks(s);

            panel.print(">     well_known_text()");
            s = gt.well_known_text();

            // GeographicTransform IF

            panel.print(">     source_gcs()");
            GeographicCoordinateSystem gcs = gt.source_gcs();

            panel.print(">     target_gcs()");
            gcs = gt.target_gcs();

            panel.print(">     forward()");
            WKSGeometry wksg = new WKSGeometry();
            wksg.point( wks_point );
            WKSGeometry wg = gt.forward( wksg );

            panel.print(">     inverse()");
            WKSGeometry wg2 = gt.inverse(wg);

        } catch (Exception e){
            result = false;
            panel.print("> ERROR !");
            e.printStackTrace();
        }
        
        return result;
    }

}